// Copyright 2016 tsuru authors. All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package healer

import (
	"time"

	"github.com/tsuru/tsuru/event"
	"github.com/tsuru/tsuru/event/eventtest"
	"gopkg.in/check.v1"
)

func (s *S) TestListHealingHistory(c *check.C) {
	evt1, err := event.NewInternal(&event.Opts{
		Target:       event.Target{Type: "node", Value: "addr1"},
		InternalKind: "healer",
	})
	c.Assert(err, check.IsNil)
	err = evt1.Done(nil)
	c.Assert(err, check.IsNil)
	time.Sleep(100 * time.Millisecond)
	evt2, err := event.NewInternal(&event.Opts{
		Target:       event.Target{Type: "container", Value: "cont1"},
		InternalKind: "healer",
	})
	c.Assert(err, check.IsNil)
	err = evt2.Done(nil)
	c.Assert(err, check.IsNil)
	evts, err := ListHealingHistory("")
	c.Assert(err, check.IsNil)
	c.Assert(evts, eventtest.EvtEquals, []HealingEvent{
		{
			ID:         evt2.UniqueID,
			StartTime:  evt2.StartTime,
			EndTime:    evt2.EndTime,
			Action:     "container-healing",
			Successful: true,
			Error:      "",
		},
		{
			ID:         evt1.UniqueID,
			StartTime:  evt1.StartTime,
			EndTime:    evt1.EndTime,
			Action:     "node-healing",
			Successful: true,
			Error:      "",
		},
	})
}
